@@ -4,13 +4,17 @@ import time
 import datetime
 import traceback
 from decimal import Decimal
+import threading
 
 import electrum
 from electrum import WalletStorage, Wallet
-from electrum.i18n import _, set_language
+from electrum_gui.kivy.i18n import _
 from electrum.contacts import Contacts
-from electrum.util import profiler
+from electrum.paymentrequest import InvoiceStore
+from electrum.util import profiler, InvalidPassword
 from electrum.plugins import run_hook
+from electrum.util import format_satoshis, format_satoshis_plain
+from electrum.paymentrequest import PR_UNPAID, PR_PAID, PR_UNKNOWN, PR_EXPIRED
 
 from kivy.app import App
 from kivy.core.window import Window
@@ -28,14 +32,17 @@ from kivy.lang import Builder
 Factory.register('InstallWizard',
                  module='electrum_gui.kivy.uix.dialogs.installwizard')
 Factory.register('InfoBubble', module='electrum_gui.kivy.uix.dialogs')
-Factory.register('ELTextInput', module='electrum_gui.kivy.uix.screens')
+Factory.register('OutputList', module='electrum_gui.kivy.uix.dialogs')
+Factory.register('OutputItem', module='electrum_gui.kivy.uix.dialogs')
 
-from kivy.core.window import Window
-Window.softinput_mode = 'below_target'
+
+
+#from kivy.core.window import Window
+#Window.softinput_mode = 'below_target'
 
 
 # delayed imports: for startup speed on android
-notification = app = ref = format_satoshis = None
+notification = app = ref = None
 util = False
 
 
@@ -47,17 +54,53 @@ from kivy.uix.screenmanager import Screen
 from kivy.uix.tabbedpanel import TabbedPanel
 from kivy.uix.label import Label
 from kivy.uix.checkbox import CheckBox
+from kivy.uix.switch import Switch
+from kivy.core.clipboard import Clipboard
 
 Factory.register('TabbedCarousel', module='electrum_gui.kivy.uix.screens')
 
+# Register fonts without this you won't be able to use bold/italic...
+# inside markup.
+from kivy.core.text import Label
+Label.register('Roboto',
+               'gui/kivy/data/fonts/Roboto.ttf',
+               'gui/kivy/data/fonts/Roboto.ttf',
+               'gui/kivy/data/fonts/Roboto-Bold.ttf',
+               'gui/kivy/data/fonts/Roboto-Bold.ttf')
 
 
-base_units = {'BTC':8, 'mBTC':5, 'uBTC':2}
+from electrum.util import base_units
+
 
 class ElectrumWindow(App):
 
     electrum_config = ObjectProperty(None)
 
+    language = StringProperty('en')
+
+    def set_URI(self, uri):
+        self.switch_to('send')
+        self.send_screen.set_URI(uri)
+
+    def on_new_intent(self, intent):
+        if intent.getScheme() != 'bitcoin':
+            return
+        uri = intent.getDataString()
+        self.set_URI(uri)
+
+    def on_language(self, instance, language):
+        Logger.info('language: {}'.format(language))
+        _.switch_lang(language)
+
+    def on_quotes(self, d):
+        #Logger.info("on_quotes")
+        pass
+
+    def on_history(self, d):
+        #Logger.info("on_history")
+        if self.history_screen:
+            self.history_screen.update()
+
     def _get_bu(self):
         return self.electrum_config.get('base_unit', 'mBTC')
 
@@ -69,32 +112,33 @@ class ElectrumWindow(App):
             self.history_screen.update()
 
     base_unit = AliasProperty(_get_bu, _set_bu)
+    status = StringProperty('')
+    fiat_unit = StringProperty('')
 
-    def _rotate_bu(self):
-        keys = sorted(base_units.keys())
-        self.base_unit = keys[ (keys.index(self.base_unit) + 1) % len(keys)]
-
-    status = StringProperty(_('Not Connected'))
+    def on_fiat_unit(self, a, b):
+        if self.history_screen:
+            self.history_screen.update()
 
     def decimal_point(self):
         return base_units[self.base_unit]
 
-    def _get_num_zeros(self):
-        try:
-            return self.electrum_config.get('num_zeros', 0)
-        except AttributeError:
-            return 0
+    def btc_to_fiat(self, amount_str):
+        if not amount_str:
+            return ''
+        rate = run_hook('exchange_rate')
+        if not rate:
+            return ''
+        fiat_amount = self.get_amount(amount_str + ' ' + self.base_unit) * rate / pow(10, 8)
+        return "{:.2f}".format(fiat_amount).rstrip('0').rstrip('.')
 
-    def _set_num_zeros(self):
-        try:
-            self.electrum_config.set_key('num_zeros', value, True)
-        except AttributeError:
-            Logger.error('Electrum: Config not available '
-                         'While trying to save value to config')
-
-    num_zeros = AliasProperty(_get_num_zeros , _set_num_zeros)
-    '''Number of zeros used while representing the value in base_unit.
-    '''
+    def fiat_to_btc(self, fiat_amount):
+        if not fiat_amount:
+            return ''
+        rate = run_hook('exchange_rate')
+        if not rate:
+            return ''
+        satoshis = int(pow(10,8) * Decimal(fiat_amount) / Decimal(rate))
+        return format_satoshis_plain(satoshis, self.decimal_point())
 
     def get_amount(self, amount_str):
         a, u = amount_str.split()
@@ -107,10 +151,6 @@ class ElectrumWindow(App):
         return int(p * x)
 
 
-    hierarchy = ListProperty([])
-    '''used to navigate with the back button.
-    '''
-
     _orientation = OptionProperty('landscape',
                                  options=('landscape', 'portrait'))
 
@@ -140,10 +180,6 @@ class ElectrumWindow(App):
     :data:`ui_mode` is a read only `AliasProperty` Defaults to 'phone'
     '''
 
-    url = StringProperty('', allownone=True)
-    '''
-    '''
-
     wallet = ObjectProperty(None)
     '''Holds the electrum wallet
 
@@ -152,16 +188,18 @@ class ElectrumWindow(App):
 
     def __init__(self, **kwargs):
         # initialize variables
-        self._clipboard = None
+        self._clipboard = Clipboard
         self.info_bubble = None
         self.qrscanner = None
         self.nfcscanner = None
         self.tabs = None
+        self.is_exit = False
 
         super(ElectrumWindow, self).__init__(**kwargs)
 
         title = _('Electrum App')
         self.electrum_config = config = kwargs.get('config', None)
+        self.language = config.get('language', 'en')
         self.network = network = kwargs.get('network', None)
         self.plugins = kwargs.get('plugins', [])
 
@@ -169,12 +207,7 @@ class ElectrumWindow(App):
 
         #self.config = self.gui_object.config
         self.contacts = Contacts(self.electrum_config)
-
-        self.bind(url=self.set_url)
-        # were we sent a url?
-        url = self.electrum_config.get('url', None)
-        if url:
-            self.set_url(url)
+        self.invoices = InvoiceStore(self.electrum_config)
 
         # create triggers so as to minimize updation a max of 2 times a sec
         self._trigger_update_wallet =\
@@ -184,12 +217,94 @@ class ElectrumWindow(App):
         self._trigger_notify_transactions = \
             Clock.create_trigger(self.notify_transactions, 5)
 
-    def set_url(self, url):
-        print "set url", url
-        url = electrum.util.parse_URI(url)
-        self.send_screen.set_qr_data(url)
+
+    def on_pr(self, pr):
+        if pr.verify(self.contacts):
+            key = self.invoices.add(pr)
+            if self.invoices_screen:
+                self.invoices_screen.update()
+            status = self.invoices.get_status(key)
+            if status == PR_PAID:
+                self.show_error("invoice already paid")
+                self.send_screen.do_clear()
+            else:
+                if pr.has_expired():
+                    self.show_error(_('Payment request has expired'))
+                else:
+                    self.switch_to('send')
+                    self.send_screen.set_request(pr)
+        else:
+            self.show_error("invoice error:" + pr.error)
+            self.send_screen.do_clear()
+
+    def on_qr(self, data):
+        if data.startswith('bitcoin:'):
+            self.set_URI(data)
+            return
+        # try to decode transaction
+        from electrum.bitcoin import base_decode
+        from electrum.transaction import Transaction
+        try:
+            text = base_decode(data, None, base=43).encode('hex')
+            tx = Transaction(text)
+        except:
+            tx = None
+        if tx:
+            self.tx_dialog(tx)
+            return
+        # show error
+        self.show_error("Unable to decode QR data")
+
+    def update_tab(self, name):
+        s = getattr(self, name + '_screen', None)
+        if s:
+            s.update()
+
+    @profiler
+    def update_tabs(self):
+        for tab in ['invoices', 'send', 'history', 'receive', 'requests']:
+            self.update_tab(tab)
+
+    def switch_to(self, name):
+        s = getattr(self, name + '_screen', None)
+        if self.send_screen is None:
+            s = self.tabs.ids[name + '_screen']
+            s.load_screen()
+        panel = self.tabs.ids.panel
+        tab = self.tabs.ids[name + '_tab']
+        panel.switch_to(tab)
+
+    def show_request(self, addr):
+        self.switch_to('receive')
+        self.receive_screen.screen.address = addr
+
+    def show_pr_details(self, req, status, is_invoice):
+        from electrum.util import format_time
+        requestor = req.get('requestor')
+        exp = req.get('exp')
+        memo = req.get('memo')
+        amount = req.get('amount')
+        popup = Builder.load_file('gui/kivy/uix/ui_screens/invoice.kv')
+        popup.is_invoice = is_invoice
+        popup.amount = amount
+        popup.requestor = requestor if is_invoice else req.get('address')
+        popup.exp = format_time(exp) if exp else ''
+        popup.description = memo if memo else ''
+        popup.signature = req.get('signature', '')
+        popup.status = status
+        txid = req.get('txid')
+        popup.tx_hash = txid or ''
+        popup.on_open = lambda: popup.ids.output_list.update(req.get('outputs', []))
+        popup.open()
+
+    def qr_dialog(self, title, data, show_text=False):
+        from uix.dialogs.qr_dialog import QRDialog
+        popup = QRDialog(title, data, show_text)
+        popup.open()
 
     def scan_qr(self, on_complete):
+        if platform != 'android':
+            return
         from jnius import autoclass
         from android import activity
         PythonActivity = autoclass('org.renpy.android.PythonActivity')
@@ -201,26 +316,14 @@ class ElectrumWindow(App):
                 if resultCode == -1: # RESULT_OK:
                     contents = intent.getStringExtra("SCAN_RESULT")
                     if intent.getStringExtra("SCAN_RESULT_FORMAT") == 'QR_CODE':
-                        uri = electrum.util.parse_URI(contents)
-                        on_complete(uri)
+                        on_complete(contents)
+                    else:
+                        self.show_error("wrong format " + intent.getStringExtra("SCAN_RESULT_FORMAT"))
         activity.bind(on_activity_result=on_qr_result)
-        PythonActivity.mActivity.startActivityForResult(intent, 0)
-
-    def show_plugins(self, plugins_list):
-        def on_checkbox_active(cb, value):
-            self.plugins.toggle_enabled(self.electrum_config, cb.name)
-        for item in self.plugins.descriptions:
-            if 'kivy' not in item.get('available_for', []):
-                continue
-            name = item.get('name')
-            label = Label(text=item.get('fullname'))
-            plugins_list.add_widget(label)
-            cb = CheckBox()
-            cb.name = name
-            p = self.plugins.get(name)
-            cb.active = (p is not None) and p.is_enabled()
-            cb.bind(active=on_checkbox_active)
-            plugins_list.add_widget(cb)
+        try:
+            PythonActivity.mActivity.startActivityForResult(intent, 0)
+        except:
+            self.show_error(_('Could not start Barcode Scanner.') + ' ' + _('Please install the Barcode Scanner app from ZXing'))
 
     def build(self):
         return Builder.load_file('gui/kivy/main.kv')
@@ -236,79 +339,62 @@ class ElectrumWindow(App):
     def on_start(self):
         ''' This is the start point of the kivy ui
         '''
+        import time
+        Logger.info('Time to on_start: {} <<<<<<<<'.format(time.clock()))
         Logger.info("dpi: {} {}".format(metrics.dpi, metrics.dpi_rounded))
         win = Window
-        win.bind(size=self.on_size,
-                    on_keyboard=self.on_keyboard)
+        win.bind(size=self.on_size, on_keyboard=self.on_keyboard)
         win.bind(on_key_down=self.on_key_down)
-
-        # Register fonts without this you won't be able to use bold/italic...
-        # inside markup.
-        from kivy.core.text import Label
-        Label.register('Roboto',
-                   'data/fonts/Roboto.ttf',
-                   'data/fonts/Roboto.ttf',
-                   'data/fonts/Roboto-Bold.ttf',
-                   'data/fonts/Roboto-Bold.ttf')
-
-        if platform == 'android':
-            # bind to keyboard height so we can get the window contents to
-            # behave the way we want when the keyboard appears.
-            win.bind(keyboard_height=self.on_keyboard_height)
-
+        win.softinput_mode = 'below_target'
         self.on_size(win, win.size)
+        self.init_ui()
+        self.load_wallet_by_name(self.electrum_config.get_wallet_path())
+        # init plugins
+        run_hook('init_kivy', self)
+        # default tab
+        self.switch_to('history')
+        # bind intent for bitcoin: URI scheme
+        if platform == 'android':
+            from android import activity
+            from jnius import autoclass
+            PythonActivity = autoclass('org.renpy.android.PythonActivity')
+            mactivity = PythonActivity.mActivity
+            self.on_new_intent(mactivity.getIntent())
+            activity.bind(on_new_intent=self.on_new_intent)
+
+        # URI passed in config
+        uri = self.electrum_config.get('url')
+        if uri:
+            self.set_URI(uri)
+
+    def load_wallet_by_name(self, wallet_path):
+        if not wallet_path:
+            return
         config = self.electrum_config
-        storage = WalletStorage(config.get_wallet_path())
-
+        storage = WalletStorage(wallet_path)
         Logger.info('Electrum: Check for existing wallet')
-
         if storage.file_exists:
             wallet = Wallet(storage)
             action = wallet.get_action()
         else:
             action = 'new'
-
         if action is not None:
             # start installation wizard
             Logger.debug('Electrum: Wallet not found. Launching install wizard')
             wizard = Factory.InstallWizard(config, self.network, storage)
-            wizard.bind(on_wizard_complete=self.on_wizard_complete)
+            wizard.bind(on_wizard_complete=lambda instance, wallet: self.load_wallet(wallet))
             wizard.run(action)
         else:
-            wallet.start_threads(self.network)
-            self.on_wizard_complete(None, wallet)
-
+            self.load_wallet(wallet)
         self.on_resume()
 
     def on_stop(self):
+        self.stop_wallet()
+
+    def stop_wallet(self):
         if self.wallet:
             self.wallet.stop_threads()
 
-    def on_back(self):
-        try:
-            self.hierarchy.pop()()
-        except IndexError:
-            # capture back button and pause app.
-            self._pause()
-
-
-    def on_keyboard_height(self, window, height):
-        win = window
-        active_widg = win.children[0]
-        if not issubclass(active_widg.__class__, Factory.Popup):
-            try:
-                active_widg = self.root.children[0]
-            except IndexError:
-                return
-
-        try:
-            fw = self._focused_widget
-        except AttributeError:
-            return
-        if height > 0 and fw.to_window(*fw.pos)[1] > height:
-            return
-        Factory.Animation(y=win.keyboard_height, d=.1).start(active_widg)
-
     def on_key_down(self, instance, key, keycode, codepoint, modifiers):
         if 'ctrl' in modifiers:
             # q=24 w=25
@@ -329,26 +415,28 @@ class ElectrumWindow(App):
         #TODO: alt+tab_number to activate the particular tab
 
     def on_keyboard(self, instance, key, keycode, codepoint, modifiers):
+        if key == 27 and self.is_exit is False:
+            self.is_exit = True
+            self.show_info(_('Press again to exit'))
+            return True
+        self.is_exit = False
         # override settings button
         if key in (319, 282): #f1/settings button on android
-            self.gui.main_gui.toggle_settings(self)
+            #self.gui.main_gui.toggle_settings(self)
             return True
 
-    def on_wizard_complete(self, instance, wallet):
-        if not wallet:
-            Logger.debug('Electrum: No Wallet set/found. Exiting...')
-            app = App.get_running_app()
-            app.show_error('Electrum: No Wallet set/found. Exiting...',
-                           exit=True)
-
-        self.init_ui()
-        self.load_wallet(wallet)
-
     def popup_dialog(self, name):
-        popup = Builder.load_file('gui/kivy/uix/ui_screens/'+name+'.kv')
-        popup.open()
-
-
+        if name == 'settings':
+            from uix.dialogs.settings import SettingsDialog
+            d = SettingsDialog(self)
+            d.open()
+        elif name == 'wallets':
+            from uix.dialogs.wallets import WalletDialog
+            d = WalletDialog()
+            d.open()
+        else:
+            popup = Builder.load_file('gui/kivy/uix/ui_screens/'+name+'.kv')
+            popup.open()
 
     @profiler
     def init_ui(self):
@@ -356,7 +444,6 @@ class ElectrumWindow(App):
         tasks of setting up the ui.
         '''
         from weakref import ref
-        set_language(self.electrum_config.get('language'))
 
         self.funds_error = False
         # setup UX
@@ -374,42 +461,50 @@ class ElectrumWindow(App):
 
         # load and focus the ui
         self.root.manager = self.root.ids['manager']
-        self.recent_activity_card = None
+
         self.history_screen = None
         self.contacts_screen = None
+        self.send_screen = None
+        self.invoices_screen = None
+        self.receive_screen = None
+        self.requests_screen = None
 
         self.icon = "icons/electrum.png"
 
         # connect callbacks
         if self.network:
-            self.network.register_callback('updated', self._trigger_update_wallet)
-            self.network.register_callback('status', self._trigger_update_status)
-            self.network.register_callback('new_transaction', self._trigger_notify_transactions)
-
-        self.wallet = None
+            interests = ['updated', 'status', 'new_transaction']
+            self.network.register_callback(self.on_network, interests)
 
+        #self.wallet = None
+        self.tabs = self.root.ids['tabs']
 
+    def on_network(self, event, *args):
+        if event == 'updated':
+            self._trigger_update_wallet()
+        elif event == 'status':
+            self._trigger_update_status()
+        elif event == 'new_transaction':
+            self._trigger_notify_transactions(*args)
 
     @profiler
     def load_wallet(self, wallet):
+        self.stop_wallet()
         self.wallet = wallet
+        self.wallet.start_threads(self.network)
         self.current_account = self.wallet.storage.get('current_account', None)
         self.update_wallet()
         # Once GUI has been initialized check if we want to announce something
         # since the callback has been called before the GUI was initialized
-        self.update_history_tab()
+        self.update_tabs()
         self.notify_transactions()
+        run_hook('load_wallet', wallet, self)
 
     def update_status(self, *dt):
         if not self.wallet:
             return
-
-        unconfirmed = ''
-        quote_text = ''
-
         if self.network is None or not self.network.is_running():
-            text = _("Offline")
-
+            self.status = _("Offline")
         elif self.network.is_connected():
             server_height = self.network.get_server_height()
             server_lag = self.network.get_local_height() - server_height
@@ -419,77 +514,31 @@ class ElectrumWindow(App):
                 self.status = _("Server lagging (%d blocks)"%server_lag)
             else:
                 c, u, x = self.wallet.get_account_balance(self.current_account)
-                text = self.format_amount(c)
-                if u:
-                    unconfirmed =  " [%s unconfirmed]" %( self.format_amount(u, True).strip())
-                if x:
-                    unmatured =  " [%s unmatured]"%(self.format_amount(x, True).strip())
-                #quote_text = self.create_quote_text(Decimal(c+u+x)/100000000, mode='symbol') or ''
-                self.status = text.strip() + ' ' + self.base_unit
+                text = self.format_amount(c+x+u)
+                self.status = str(text.strip() + ' ' + self.base_unit)
         else:
             self.status = _("Not connected")
 
-        return
-
-        print self.root.manager.ids
-
-        #try:
-        status_card = self.root.main_screen.ids.tabs.ids.\
-                      screen_dashboard.ids.status_card
-        #except AttributeError:
-        #    return
-
-        status_card.quote_text = quote_text.strip()
-        status_card.uncomfirmed = unconfirmed.strip()
-
-
     def get_max_amount(self):
-        from electrum.util import format_satoshis_plain
         inputs = self.wallet.get_spendable_coins(None)
-        amount, fee = self.wallet.get_max_amount(self.electrum_config, inputs, None)
+        addr = str(self.send_screen.screen.address) or self.wallet.dummy_address()
+        amount, fee = self.wallet.get_max_amount(self.electrum_config, inputs, addr, None)
         return format_satoshis_plain(amount, self.decimal_point())
 
-    def update_amount(self, amount, c):
-        if c == '<':
-            return amount[:-1]
-        if c == '.' and amount == '':
-            return '0.'
-        if c == '0' and amount == '0':
-            return '0'
-        try:
-            Decimal(amount+c)
-            amount += c
-        except:
-            pass
-        return amount
-
     def format_amount(self, x, is_diff=False, whitespaces=False):
-        from electrum.util import format_satoshis
-        return format_satoshis(x, is_diff, self.num_zeros,
-                               self.decimal_point(), whitespaces)
+        return format_satoshis(x, is_diff, 0, self.decimal_point(), whitespaces)
+
+    def format_amount_and_units(self, x):
+        return format_satoshis_plain(x, self.decimal_point()) + ' ' + self.base_unit
 
     @profiler
     def update_wallet(self, *dt):
         self._trigger_update_status()
-        if self.wallet.up_to_date or not self.network or not self.network.is_connected():
-            self.update_history_tab()
-            self.update_contacts_tab()
-
-
-    @profiler
-    def update_history_tab(self, see_all=False):
-        if self.history_screen:
-            self.history_screen.update(see_all)
-
-    def update_contacts_tab(self):
-        if self.contacts_screen:
-            self.contacts_screen.update()
-
+        #if self.wallet.up_to_date or not self.network or not self.network.is_connected():
+        self.update_tabs()
 
     @profiler
     def notify_transactions(self, *dt):
-        '''
-        '''
         if not self.network or not self.network.is_connected():
             return
         # temporarily disabled for merge
@@ -523,14 +572,6 @@ class ElectrumWindow(App):
                               format(txs=tx_amount, amount=self.format_amount(v),
                                      unit=self.base_unit))
 
-    def copy(self, text):
-        ''' Copy provided text to clipboard
-        '''
-        if not self._clipboard:
-            from kivy.core.clipboard import Clipboard
-            self._clipboard = Clipboard
-        self._clipboard.put(text, 'text/plain')
-
     def notify(self, message):
         try:
             global notification, os
@@ -545,8 +586,6 @@ class ElectrumWindow(App):
             Logger.Error('Notification: needs plyer; `sudo pip install plyer`')
 
     def on_pause(self):
-        '''
-        '''
         # pause nfc
         if self.qrscanner:
             self.qrscanner.stop()
@@ -555,8 +594,6 @@ class ElectrumWindow(App):
         return True
 
     def on_resume(self):
-        '''
-        '''
         if self.qrscanner and qrscanner.get_parent_window():
             self.qrscanner.start()
         if self.nfcscanner:
@@ -566,103 +603,19 @@ class ElectrumWindow(App):
         width, height = value
         self._orientation = 'landscape' if width > height else 'portrait'
         self._ui_mode = 'tablet' if min(width, height) > inch(3.51) else 'phone'
-        #Logger.info("size: {} {}".format(width, height))
-        #Logger.info('orientation: {}'.format(self._orientation))
-        #Logger.info('ui_mode: {}'.format(self._ui_mode))
 
-    def save_new_contact(self, address, label):
-        address = unicode(address)
-        label = unicode(label)
-        global is_valid
-        if not is_valid:
-            from electrum.bitcoin import is_valid
-
-        if is_valid(address):
-            if label:
-                self.set_label(address, text=label)
-            self.wallet.add_contact(address)
-            self.update_contacts_tab()
-            self.update_history_tab()
+    def on_ref_label(self, label, touch):
+        if label.touched:
+            label.touched = False
+            self.qr_dialog(label.name, label.data, True)
         else:
-            self.show_error(_('Invalid Address'))
-
-    def send_payment(self, address, amount=0, label='', message=''):
-        tabs = self.tabs
-        screen_send = tabs.ids.screen_send
-
-        if label and self.wallet.labels.get(address) != label:
-            #if self.question('Give label "%s" to address %s ?'%(label,address)):
-            if address not in self.wallet.addressbook and not self.wallet.  is_mine(address):
-                self.wallet.addressbook.append(address)
-            self.wallet.set_label(address, label)
-
-        # switch_to the send screen
-        tabs.ids.panel.switch_to(tabs.ids.tab_send)
-
-        label = self.wallet.labels.get(address)
-        m_addr = label + '  <'+ address +'>' if label else address
-
-        # populate
-        def set_address(*l):
-            content = screen_send.ids
-            content.payto_e.text = m_addr
-            content.message_e.text = message
-            if amount:
-                content.amount_e.text = amount
-
-        # wait for screen to load
-        Clock.schedule_once(set_address, .5)
+            label.touched = True
+            self._clipboard.copy(label.data)
+            Clock.schedule_once(lambda dt: self.show_info(_('Text copied to clipboard.\nTap again to display it as QR code.')))
 
     def set_send(self, address, amount, label, message):
         self.send_payment(address, amount=amount, label=label, message=message)
 
-    def prepare_for_payment_request(self):
-        tabs = self.tabs
-        screen_send = tabs.ids.screen_send
-
-        # switch_to the send screen
-        tabs.ids.panel.switch_to(tabs.ids.tab_send)
-
-        content = screen_send.ids
-        if content:
-            self.set_frozen(content, False)
-        screen_send.screen_label.text = _("please wait...")
-        return True
-
-    def payment_request_ok(self):
-        tabs = self.tabs
-        screen_send = tabs.ids.screen_send
-
-        # switch_to the send screen
-        tabs.ids.panel.switch_to(tabs.ids.tab_send)
-
-        self.set_frozen(content, True)
-
-        screen_send.ids.payto_e.text = self.gui_object.payment_request.domain
-        screen_send.ids.amount_e.text = self.format_amount(self.gui_object.payment_request.get_amount())
-        screen_send.ids.message_e.text = self.gui_object.payment_request.memo
-
-        # wait for screen to load
-        Clock.schedule_once(set_address, .5)
-
-    def set_frozen(self, entry, frozen):
-        if frozen:
-            entry.disabled = True
-            Factory.Animation(opacity=0).start(content)
-        else:
-            entry.disabled = False
-            Factory.Animation(opacity=1).start(content)
-
-    def payment_request_error(self):
-        tabs = self.tabs
-        screen_send = tabs.ids.screen_send
-
-        # switch_to the send screen
-        tabs.ids.panel.switch_to(tabs.ids.tab_send)
-
-        self.do_clear()
-        self.show_info(self.gui_object.payment_request.error)
-
     def show_error(self, error, width='200dp', pos=None, arrow_pos=None,
         exit=False, icon='atlas://gui/kivy/theming/light/error', duration=0,
         modal=False):
@@ -676,7 +629,7 @@ class ElectrumWindow(App):
         exit=False, duration=0, modal=False):
         ''' Show a Info Message Bubble.
         '''
-        self.show_error(error, icon='atlas://gui/kivy/theming/light/error',
+        self.show_error(error, icon='atlas://gui/kivy/theming/light/important',
             duration=duration, modal=modal, exit=exit, pos=pos,
             arrow_pos=arrow_pos)
 
@@ -727,43 +680,105 @@ class ElectrumWindow(App):
             info_bubble.background_image = 'atlas://data/images/defaulttheme/bubble'
         info_bubble.message = text
         if not pos:
-                pos = (win.center[0], win.center[1] - (info_bubble.height/2))
+            pos = (win.center[0], win.center[1] - (info_bubble.height/2))
         info_bubble.show(pos, duration, width, modal=modal, exit=exit)
 
-    def tx_dialog(self, tx_hash):
-        popup = Builder.load_file('gui/kivy/uix/ui_screens/transaction.kv')
-        popup.tx_hash = tx_hash
-        popup.open()
+    def tx_dialog(self, tx):
+        from uix.dialogs.tx_dialog import TxDialog
+        d = TxDialog(self, tx)
+        d.open()
 
-    def amount_dialog(self, label, callback, show_max):
-        popup = Builder.load_file('gui/kivy/uix/ui_screens/amount.kv')
-        but_max = popup.ids.but_max
-        if not show_max:
-            but_max.disabled = True
-            but_max.opacity = 0
+    def sign_tx(self, *args):
+        threading.Thread(target=self._sign_tx, args=args).start()
+
+    def _sign_tx(self, tx, password, on_success, on_failure):
+        try:
+            self.wallet.sign_transaction(tx, password)
+        except InvalidPassword:
+            Clock.schedule_once(lambda dt: on_failure(_("Invalid PIN")))
+            return
+        Clock.schedule_once(lambda dt: on_success(tx))
+
+    def _broadcast_thread(self, tx, on_complete):
+        ok, txid = self.network.broadcast(tx)
+        Clock.schedule_once(lambda dt: on_complete(ok, txid))
+
+    def broadcast(self, tx, pr=None):
+        def on_complete(ok, txid):
+            self.show_info(txid)
+            if ok and pr:
+                pr.set_paid(tx.hash())
+                self.invoices.save()
+                self.update_tab('invoices')
+
+        if self.network and self.network.is_connected():
+            self.show_info(_('Sending'))
+            threading.Thread(target=self._broadcast_thread, args=(tx, on_complete)).start()
         else:
-            but_max.disabled = False
-            but_max.opacity = 1
+            self.show_info(_('Cannot broadcast transaction') + ':\n' + _('Not connected'))
 
-        if label.text != label.default_text:
-            a, u = label.text.split()
+    def description_dialog(self, screen):
+        from uix.dialogs.label_dialog import LabelDialog
+        text = screen.message
+        def callback(text):
+            screen.message = text
+        d = LabelDialog(_('Enter description'), text, callback)
+        d.open()
+
+    @profiler
+    def amount_dialog(self, screen, show_max):
+        from uix.dialogs.amount_dialog import AmountDialog
+        amount = screen.amount
+        if amount:
+            amount, u = str(amount).split()
             assert u == self.base_unit
-            popup.ids.a.amount = a
-        def cb():
-            o = popup.ids.a.text
-            label.text = o if o else label.default_text
-            if callback:
-                callback()
-        popup.on_dismiss = cb
+        def cb(amount):
+            screen.amount = amount
+        popup = AmountDialog(show_max, amount, cb)
         popup.open()
 
-    def password_dialog(self, f, args):
+    def protected(self, msg, f, args):
         if self.wallet.use_encryption:
-            popup = Builder.load_file('gui/kivy/uix/ui_screens/password.kv')
-            def callback():
-                pw = popup.ids.text_input.text
-                Clock.schedule_once(lambda x: apply(f, args + (pw,)), 0.5)
-            popup.on_dismiss = callback
-            popup.open()
+            self.password_dialog(msg, f, args)
         else:
             apply(f, args + (None,))
+
+    def show_seed(self, label):
+        self.protected(_("Enter your PIN code in order to decrypt your seed"), self._show_seed, (label,))
+
+    def _show_seed(self, label, password):
+        try:
+            seed = self.wallet.get_seed(password)
+        except:
+            self.show_error("Invalid PIN")
+            return
+        label.text = _('Seed') + ':\n' + seed
+
+    def change_password(self):
+        self.protected(_("Changing PIN code.") + '\n' + _("Enter your current PIN:"), self._change_password, ())
+
+    def _change_password(self, old_password):
+        if self.wallet.use_encryption:
+            try:
+                self.wallet.check_password(old_password)
+            except InvalidPassword:
+                self.show_error("Invalid PIN")
+                return
+        self.password_dialog(_('Enter new PIN'), self._change_password2, (old_password,))
+
+    def _change_password2(self, old_password, new_password):
+        self.password_dialog(_('Confirm new PIN'), self._change_password3, (old_password, new_password))
+
+    def _change_password3(self, old_password, new_password, confirmed_password):
+        if new_password == confirmed_password:
+            self.wallet.update_password(old_password, new_password)
+        else:
+            self.show_error("PIN numbers do not match")
+
+    def password_dialog(self, msg, f, args):
+        from uix.dialogs.password_dialog import PasswordDialog
+        def callback(pw):
+            Clock.schedule_once(lambda x: apply(f, args + (pw,)), 0.1)
+        popup = PasswordDialog(msg, callback)
+        popup.open()
+
